-module(gleatfy).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([server/2, login/2, topic/2, message/2, title/2, priority/2, tags/2, delay/2, click_url/2, attachment_url/2, attachment_filename/2, icon_url/2, email/2, call/2, actions/2, without_message_cache/1, without_firebase/1, send/2, new/0]).
-export_type([error/1, priority/0, login/0, action/0, message/0, builder/0]).

-type error(HHB) :: {invalid_server_url, binary()} |
    {invalid_topic, binary()} |
    {client_error, HHB} |
    {invalid_server_response, binary()} |
    {server_error, integer(), integer(), binary()}.

-type priority() :: very_high | high | normal | low | very_low.

-type login() :: {basic, binary(), binary()} | {token, binary()}.

-type action() :: {view, binary(), binary(), boolean()} |
    {http, binary(), gleam@http@request:request(binary()), boolean()} |
    {broadcast, binary(), binary(), list({binary(), binary()}), boolean()}.

-type message() :: {text, binary()} | {markdown, binary()}.

-opaque builder() :: {builder,
        gleam@option:option(login()),
        binary(),
        binary(),
        gleam@option:option(message()),
        boolean(),
        gleam@option:option(binary()),
        gleam@option:option(priority()),
        gleam@option:option(list(binary())),
        gleam@option:option(binary()),
        gleam@option:option(binary()),
        gleam@option:option(binary()),
        gleam@option:option(binary()),
        gleam@option:option(binary()),
        gleam@option:option(binary()),
        gleam@option:option(binary()),
        gleam@option:option(list(action())),
        boolean(),
        boolean()}.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 113).
-spec server(builder(), binary()) -> builder().
server(Builder, Server) ->
    erlang:setelement(3, Builder, Server).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 120).
-spec login(builder(), login()) -> builder().
login(Builder, Login) ->
    erlang:setelement(2, Builder, {some, Login}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 126).
-spec topic(builder(), binary()) -> builder().
topic(Builder, Topic) ->
    erlang:setelement(4, Builder, Topic).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 144).
-spec message(builder(), message()) -> builder().
message(Builder, Message) ->
    erlang:setelement(5, Builder, {some, Message}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 151).
-spec title(builder(), binary()) -> builder().
title(Builder, Title) ->
    erlang:setelement(7, Builder, {some, Title}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 158).
-spec priority(builder(), priority()) -> builder().
priority(Builder, Priority) ->
    erlang:setelement(8, Builder, {some, Priority}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 165).
-spec tags(builder(), list(binary())) -> builder().
tags(Builder, Tags) ->
    erlang:setelement(9, Builder, {some, Tags}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 172).
-spec delay(builder(), binary()) -> builder().
delay(Builder, Delay) ->
    erlang:setelement(10, Builder, {some, Delay}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 179).
-spec click_url(builder(), binary()) -> builder().
click_url(Builder, Click_url) ->
    erlang:setelement(15, Builder, {some, Click_url}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 186).
-spec attachment_url(builder(), binary()) -> builder().
attachment_url(Builder, Attachment_url) ->
    erlang:setelement(13, Builder, {some, Attachment_url}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 193).
-spec attachment_filename(builder(), binary()) -> builder().
attachment_filename(Builder, Filename) ->
    erlang:setelement(14, Builder, {some, Filename}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 200).
-spec icon_url(builder(), binary()) -> builder().
icon_url(Builder, Icon_url) ->
    erlang:setelement(16, Builder, {some, Icon_url}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 207).
-spec email(builder(), binary()) -> builder().
email(Builder, Email) ->
    erlang:setelement(12, Builder, {some, Email}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 214).
-spec call(builder(), binary()) -> builder().
call(Builder, Number) ->
    erlang:setelement(11, Builder, {some, Number}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 258).
-spec actions(builder(), list(action())) -> builder().
actions(Builder, Actions) ->
    erlang:setelement(17, Builder, {some, Actions}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 265).
-spec without_message_cache(builder()) -> builder().
without_message_cache(Builder) ->
    erlang:setelement(18, Builder, true).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 272).
-spec without_firebase(builder()) -> builder().
without_firebase(Builder) ->
    erlang:setelement(19, Builder, true).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 318).
-spec decode_success(binary()) -> {ok, binary()} | {error, error(any())}.
decode_success(Body) ->
    _pipe = Body,
    _pipe@1 = gleam@json:decode(
        _pipe,
        gleam@dynamic:decode1(
            fun gleam@function:identity/1,
            gleam@dynamic:field(<<"id"/utf8>>, fun gleam@dynamic:string/1)
        )
    ),
    gleam@result:replace_error(_pipe@1, {invalid_server_response, Body}).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 327).
-spec decode_error(binary()) -> {ok, binary()} | {error, error(any())}.
decode_error(Body) ->
    Decode_error = gleam@dynamic:decode3(
        fun(Field@0, Field@1, Field@2) -> {server_error, Field@0, Field@1, Field@2} end,
        gleam@dynamic:field(<<"code"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(<<"http"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(<<"error"/utf8>>, fun gleam@dynamic:string/1)
    ),
    case gleam@json:decode(Body, Decode_error) of
        {ok, Err} ->
            {error, Err};

        {error, _} ->
            {error, {invalid_server_response, Body}}
    end.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 360).
-spec optional(
    list({binary(), gleam@json:json()}),
    binary(),
    gleam@option:option(HIA),
    fun((HIA) -> gleam@json:json())
) -> list({binary(), gleam@json:json()}).
optional(Params, Name, Value, Fun) ->
    case Value of
        none ->
            Params;

        {some, V} ->
            [{Name, Fun(V)} | Params]
    end.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 372).
-spec json_message(
    list({binary(), gleam@json:json()}),
    gleam@option:option(message())
) -> list({binary(), gleam@json:json()}).
json_message(Params, Message) ->
    case Message of
        none ->
            Params;

        {some, {text, Value}} ->
            [{<<"message"/utf8>>, gleam@json:string(Value)} | Params];

        {some, {markdown, Value@1}} ->
            [{<<"message"/utf8>>, gleam@json:string(Value@1)},
                {<<"markdown"/utf8>>, gleam@json:bool(true)} |
                Params]
    end.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 387).
-spec json_priority(priority()) -> gleam@json:json().
json_priority(Priority) ->
    gleam@json:int(case Priority of
            very_high ->
                5;

            high ->
                4;

            normal ->
                3;

            low ->
                2;

            very_low ->
                1
        end).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 397).
-spec json_string_map(list({binary(), binary()})) -> gleam@json:json().
json_string_map(Items) ->
    _pipe = Items,
    _pipe@1 = gleam@list:map(
        _pipe,
        fun(_capture) ->
            gleam@pair:map_second(_capture, fun gleam@json:string/1)
        end
    ),
    gleam@json:object(_pipe@1).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 414).
-spec json_base_action(
    binary(),
    binary(),
    boolean(),
    list({binary(), gleam@json:json()})
) -> gleam@json:json().
json_base_action(Action, Label, Clear_after, Properties) ->
    _pipe = [{<<"action"/utf8>>, gleam@json:string(Action)},
        {<<"label"/utf8>>, gleam@json:string(Label)},
        {<<"clear"/utf8>>, gleam@json:bool(Clear_after)} |
        Properties],
    gleam@json:object(_pipe).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 429).
-spec json_view_action(binary(), binary(), boolean()) -> gleam@json:json().
json_view_action(Label, Url, Clear_after) ->
    json_base_action(
        <<"view"/utf8>>,
        Label,
        Clear_after,
        [{<<"url"/utf8>>, gleam@json:string(Url)}]
    ).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 433).
-spec json_http_action(
    binary(),
    gleam@http@request:request(binary()),
    boolean()
) -> gleam@json:json().
json_http_action(Label, Request, Clear_after) ->
    json_base_action(
        <<"broadcast"/utf8>>,
        Label,
        Clear_after,
        [{<<"method"/utf8>>,
                begin
                    _pipe = erlang:element(2, Request),
                    _pipe@1 = gleam@http:method_to_string(_pipe),
                    gleam@json:string(_pipe@1)
                end},
            {<<"url"/utf8>>,
                begin
                    _pipe@2 = Request,
                    _pipe@3 = gleam@http@request:to_uri(_pipe@2),
                    _pipe@4 = gleam@uri:to_string(_pipe@3),
                    gleam@json:string(_pipe@4)
                end},
            {<<"headers"/utf8>>, json_string_map(erlang:element(3, Request))},
            {<<"body"/utf8>>, gleam@json:string(erlang:element(4, Request))}]
    ).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 446).
-spec json_broadcast_action(
    binary(),
    binary(),
    list({binary(), binary()}),
    boolean()
) -> gleam@json:json().
json_broadcast_action(Label, Intent, Extras, Clear_after) ->
    json_base_action(
        <<"broadcast"/utf8>>,
        Label,
        Clear_after,
        [{<<"intent"/utf8>>, gleam@json:string(Intent)},
            {<<"extras"/utf8>>, json_string_map(Extras)}]
    ).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 403).
-spec json_action(action()) -> gleam@json:json().
json_action(Action) ->
    case Action of
        {view, Label, Url, Clear_after} ->
            json_view_action(Label, Url, Clear_after);

        {http, Label@1, Request, Clear_after@1} ->
            json_http_action(Label@1, Request, Clear_after@1);

        {broadcast, Label@2, Intent, Extras, Clear_after@2} ->
            json_broadcast_action(Label@2, Intent, Extras, Clear_after@2)
    end.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 342).
-spec request_body(builder()) -> binary().
request_body(Builder) ->
    _pipe = [{<<"topic"/utf8>>, gleam@json:string(erlang:element(4, Builder))}],
    _pipe@1 = json_message(_pipe, erlang:element(5, Builder)),
    _pipe@2 = optional(
        _pipe@1,
        <<"title"/utf8>>,
        erlang:element(7, Builder),
        fun gleam@json:string/1
    ),
    _pipe@3 = optional(
        _pipe@2,
        <<"attach"/utf8>>,
        erlang:element(13, Builder),
        fun gleam@json:string/1
    ),
    _pipe@4 = optional(
        _pipe@3,
        <<"filename"/utf8>>,
        erlang:element(14, Builder),
        fun gleam@json:string/1
    ),
    _pipe@5 = optional(
        _pipe@4,
        <<"priority"/utf8>>,
        erlang:element(8, Builder),
        fun json_priority/1
    ),
    _pipe@6 = optional(
        _pipe@5,
        <<"tags"/utf8>>,
        erlang:element(9, Builder),
        fun(_capture) -> gleam@json:array(_capture, fun gleam@json:string/1) end
    ),
    _pipe@7 = optional(
        _pipe@6,
        <<"delay"/utf8>>,
        erlang:element(10, Builder),
        fun gleam@json:string/1
    ),
    _pipe@8 = optional(
        _pipe@7,
        <<"call"/utf8>>,
        erlang:element(11, Builder),
        fun gleam@json:string/1
    ),
    _pipe@9 = optional(
        _pipe@8,
        <<"email"/utf8>>,
        erlang:element(12, Builder),
        fun gleam@json:string/1
    ),
    _pipe@10 = optional(
        _pipe@9,
        <<"click"/utf8>>,
        erlang:element(15, Builder),
        fun gleam@json:string/1
    ),
    _pipe@11 = optional(
        _pipe@10,
        <<"icon"/utf8>>,
        erlang:element(16, Builder),
        fun gleam@json:string/1
    ),
    _pipe@12 = optional(
        _pipe@11,
        <<"actions"/utf8>>,
        erlang:element(17, Builder),
        fun(_capture@1) -> gleam@json:array(_capture@1, fun json_action/1) end
    ),
    _pipe@13 = gleam@json:object(_pipe@12),
    gleam@json:to_string(_pipe@13).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 458).
-spec set_login(
    gleam@http@request:request(binary()),
    gleam@option:option(login())
) -> gleam@http@request:request(binary()).
set_login(Request, Login) ->
    case Login of
        none ->
            Request;

        {some, {token, Token}} ->
            gleam@http@request:set_header(
                Request,
                <<"authorization"/utf8>>,
                <<"Bearer "/utf8, Token/binary>>
            );

        {some, {basic, User, Pass}} ->
            gleam@http@request:set_header(
                Request,
                <<"authorization"/utf8>>,
                <<"Basic "/utf8,
                    (gleam_stdlib:bit_array_base64_encode(
                        gleam_stdlib:identity(
                            <<<<User/binary, ":"/utf8>>/binary, Pass/binary>>
                        ),
                        false
                    ))/binary>>
            )
    end.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 476).
-spec set_message_cache(gleam@http@request:request(binary()), boolean()) -> gleam@http@request:request(binary()).
set_message_cache(Request, Without_message_cache) ->
    case Without_message_cache of
        true ->
            _pipe = Request,
            gleam@http@request:set_header(
                _pipe,
                <<"cache"/utf8>>,
                <<"no"/utf8>>
            );

        false ->
            Request
    end.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 486).
-spec set_firebase(gleam@http@request:request(binary()), boolean()) -> gleam@http@request:request(binary()).
set_firebase(Request, Without_firebase) ->
    case Without_firebase of
        true ->
            _pipe = Request,
            gleam@http@request:set_header(
                _pipe,
                <<"firebase"/utf8>>,
                <<"no"/utf8>>
            );

        false ->
            Request
    end.

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 298).
-spec request(builder()) -> {ok, gleam@http@request:request(binary())} |
    {error, error(any())}.
request(Builder) ->
    gleam@result:'try'(
        begin
            _pipe = gleam@http@request:to(erlang:element(3, Builder)),
            gleam@result:replace_error(
                _pipe,
                {invalid_server_url, erlang:element(3, Builder)}
            )
        end,
        fun(Request) ->
            gleam@bool:guard(
                erlang:element(4, Builder) =:= <<""/utf8>>,
                {error, {invalid_topic, erlang:element(4, Builder)}},
                fun() -> _pipe@1 = Request,
                    _pipe@2 = gleam@http@request:set_body(
                        _pipe@1,
                        request_body(Builder)
                    ),
                    _pipe@3 = gleam@http@request:set_method(_pipe@2, post),
                    _pipe@4 = set_login(_pipe@3, erlang:element(2, Builder)),
                    _pipe@5 = set_message_cache(
                        _pipe@4,
                        erlang:element(18, Builder)
                    ),
                    _pipe@6 = set_firebase(_pipe@5, erlang:element(19, Builder)),
                    {ok, _pipe@6} end
            )
        end
    ).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 285).
-spec send(
    builder(),
    fun((gleam@http@request:request(binary())) -> {ok,
            gleam@http@response:response(binary())} |
        {error, HHG})
) -> {ok, binary()} | {error, error(HHG)}.
send(Builder, Send_fn) ->
    gleam@result:'try'(
        request(Builder),
        fun(Request) ->
            gleam@result:'try'(
                begin
                    _pipe = Send_fn(Request),
                    gleam@result:map_error(
                        _pipe,
                        fun(Field@0) -> {client_error, Field@0} end
                    )
                end,
                fun(Response) -> case Response of
                        {response, 200, _, Body} ->
                            decode_success(Body);

                        {response, _, _, Body@1} ->
                            decode_error(Body@1)
                    end end
            )
        end
    ).

-file("/home/lx/dev/prv/gleatfy/src/gleatfy.gleam", 88).
-spec new() -> builder().
new() ->
    {builder,
        none,
        <<"https://ntfy.sh"/utf8>>,
        <<""/utf8>>,
        none,
        false,
        none,
        none,
        none,
        none,
        none,
        none,
        none,
        none,
        none,
        none,
        none,
        false,
        false}.
